import java.io.*;

public aspect WriterTest {

   /*
	* This aspect checks if a writer is created for an output stream,
	* neither the stream nor the writer were closed when writing to either of the two.   
	*/
	
	pointcut createWriter(OutputStream os):
		call(Writer+.new(OutputStream+,..)) && args(os,..);
	
	tracematch(Writer w, OutputStream os) {
		
		sym create after returning(w): createWriter(os) && !within(dacapo..*);
		
		sym closeW before: call(* close()) && target(w) && !within(dacapo..*);
		sym closeO before: call(* close()) && target(os) && !within(dacapo..*);
		
		sym writeW before: (call(* write(..)) || call(* print*(..))) && target(w) && !within(dacapo..*);
		sym writeO before: call(* write(..)) && target(os) && !within(dacapo..*);
		
		create (writeW|writeO)* (closeW|closeO)+ (writeW|writeO) {
            if(System.getProperty("TMTEST_ACTIVE")!=null) {
                System.err.println("##### Tracematch "+getClass().getName()+" matched!");
            }
		}
	}

}
